/* IBM Confidential
 *
 * OCO Source Materials
 *
 * 5724-E11
 * 5724-E26
 * (C) Copyright IBM Corp. 2002, 2005
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been deposited
 * with the U.S. Copyright office
 */

package jmsnode;

//Import required classes.

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageEOFException;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.BytesMessage;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;

import com.ibm.mq.jms.JMSC;
import com.ibm.mq.jms.MQTopicConnectionFactory;

//-----------------------------------------------------------------------------
//IPSubscriber class
//-----------------------------------------------------------------------------
public class IPSubscriber {

	public MQTopicConnectionFactory ippsFactory = new MQTopicConnectionFactory();
	TopicConnection ipConn = null;
	private String ourtopic = null;
	
	//-----------------------------------------------------------------------------
	// Method: IPSubscriber.IPSubscriber
	//-----------------------------------------------------------------------------
	/**
	 * <p>This is the constructor.
	 * 
	 * @param topic
	 */
	//-----------------------------------------------------------------------------	
	public IPSubscriber(String topic) {
		ourtopic = topic;
	}
	
	//-----------------------------------------------------------------------------
	// Method: IPSubscriber.start
	//-----------------------------------------------------------------------------
	/**
	 * <p>This method starts the IP connection to the Broker's Publication engine.
	 */
	//-----------------------------------------------------------------------------	
	public void start(){
		try {
			ippsFactory.setTransportType(JMSC.MQJMS_TP_DIRECT_TCPIP);
			ippsFactory.setHostName("localhost");
			ippsFactory.setPort(1599);
			
			ipConn = ippsFactory.createTopicConnection();
			ipConn.start();
			
			subscribe();
			
		} catch (JMSException e1) {
			System.out.println("IP Subscriber:\tError creating IP Topic Connection");
			e1.printStackTrace();
		}		
	}
	
	//-----------------------------------------------------------------------------
	// Method: IPSubscriber.subscribe
	//-----------------------------------------------------------------------------
	/**
	 * <p>This method takes a topic string and registers it.
	 * <br>It then enters the receive loop and waits for messages.
	 * <br>When a message arrives it prints it to the console.
	 */
	//-----------------------------------------------------------------------------
	public void subscribe() {
        
		try {
			TopicSession topicsession = ipConn.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
			Topic topic = topicsession.createTopic(ourtopic);
	        System.out.println("IP Subscriber:\tSubscribing on topic: " + ourtopic);
	        TopicSubscriber topicsubscriber = topicsession.createSubscriber(topic);
	        Message message = null;
	        
	        try {
	        	message = (Message)topicsubscriber.receive();
	        } catch (MessageEOFException eof) {}
	
	        if (message instanceof BytesMessage)
	        {
	        	System.out.println("IP Subscriber:\tReceived Bytes Message");
	        	BytesMessage bytesmessage = (BytesMessage)message;
	        	byte msgbytes[] = new byte[1000];
	        	int i = bytesmessage.readBytes(msgbytes);
	        	System.out.println("IP Subscriber:\tMessage Text = " + new String(msgbytes, 0, i));
	        	System.out.println("IP Subscriber:\tMessage Destination = " + bytesmessage.getJMSDestination());
	        }
	        if(message instanceof TextMessage)
	        {
	            System.out.println("IP Subscriber:\tReceived Text Message");
	            TextMessage textmessage = (TextMessage)message;
	            System.out.println("IP Subscriber:\tMessage Text = " + textmessage.getText());
	            System.out.println("IP Subscriber:\tMessage Destination = " + textmessage.getJMSDestination());
	        } else
	        if(message == null)
	            System.out.println("IP Subscriber:\tERROR: No Message received");
			
		} catch (JMSException e1) {
			System.out.println("IP Subscriber:\tCannot subscribe");
			e1.printStackTrace();
		}
	}
	
	//-----------------------------------------------------------------------------
	// Method: IPSubscriber.main
	//-----------------------------------------------------------------------------
	/**
	 * <p>This method starts an IPSubscriber.
	 */
	//-----------------------------------------------------------------------------
	public static void main(String[] args) {
		IPSubscriber sub = new IPSubscriber("update/invoices");
		sub.start();
	}
}
